#include <errno.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <limits.h>
#include <ctype.h>

/*
  Run a program with program name and args specified by argv.
  The exec'ed program uses inpipe[READ_DESC] and outpipe[WRITE_DESC]
  as stdin and stdout, resp.
 */

#define READ_DESC 0
#define WRITE_DESC 1
#define MAXARGS 512

pid_t
forkexec(char **argv, char **envp, int outpipe[2], int inpipe[2])

{
    extern char **environ;
    pid_t pid;

    if ((pid = fork()) == 0) {

	/* In the child process */

	if (inpipe) {
	    if (inpipe[READ_DESC] != 0) {
		close (0);
		if (dup (inpipe[READ_DESC]) < 0) {
		    fprintf (stderr, 
			     "forkexec: dup (inpipe[%d]=%d) failed: %s\n",
			     READ_DESC, inpipe[READ_DESC], 
			     strerror (errno));
		    
		    return (-1);
		}
	    }
	    if (inpipe[WRITE_DESC])
		close(inpipe[WRITE_DESC]);
	}

	if (outpipe) {
	    if (outpipe[WRITE_DESC] != 1) {
		close(1);
		if (dup (outpipe[WRITE_DESC]) < 0) {
			fprintf (stderr, 
				 "forkexec: dup(outpipe[%d]=%d) failed: %s",
				 WRITE_DESC, outpipe[WRITE_DESC], 
				 strerror (errno));
		    return (-1);
		}
	    }
	    if (outpipe[READ_DESC]) 
		close (outpipe[READ_DESC]);
	}
	
	environ = envp;
	execve(argv[0], argv, envp);
	fprintf (stderr, 
		 "forkexec: execv of \"%s\" failed: %s\n", 
		 (argv[0]? argv[0] : "--NULL--"), strerror (errno));
	return (-1);
	
    } else if (pid == -1) {

        fprintf (stderr, "forkexec: fork() failed: %s", strerror (errno));
	return (-1);

    } else {
	/* In the parent process */
	
	if (outpipe && outpipe[WRITE_DESC]) {
	    close(outpipe[WRITE_DESC]);
	}
    }

    return (pid);
}

pid_t
forkexec_cmd (char *cmd, char **envp, int outpipe[2], int inpipe[2])

{
    char *argv[MAXARGS];
    int i;
    char *cpcmd;
    char *p, *q;
    pid_t retval;
    int should_break = 0;

    cpcmd  = strdup (cmd);

    for (p = cpcmd, q = cpcmd, i = 0; *p && !should_break &&
	 (i < MAXARGS); p = q , i++) {

	while ((*q) && !isspace (*q)) q++;
	if (!*q) {
	    should_break = 1;
	} else {
	    should_break = 0;
	    *q = '\0';
	}
	argv[i] = p;
	q++;
    }

    argv[i] = NULL;
    retval = forkexec (argv, envp, outpipe, inpipe);
    free (cpcmd);
    return retval;
}
