/*
    Copyright (C) 1999 Paul Barton-Davis 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: selector.cc,v 1.4 1999/06/20 13:48:53 stef Exp $
*/

#include <gdk/gdk.h>

#include "selector.h"

Gtk_Selector::Gtk_Selector (void (*func)(Gtk_CList &, void *), void *arg,
			    size_t ncols, bool hscroll, bool vscroll,
			    const gchar **titles) 
	: list (ncols, titles)

{
	scroll.add (list);

	scroll.set_policy (
		hscroll ? 
		GTK_POLICY_AUTOMATIC : GTK_POLICY_NEVER,
		vscroll ?
		GTK_POLICY_AUTOMATIC : GTK_POLICY_NEVER);

	pack_start (scroll, true, true);

	connect_to_method (list.select_row, this, &Gtk_Selector::row_selected);
	connect_to_method (show, this, &Gtk_Selector::rescan);
	connect_to_method (update_contents, this, &Gtk_Selector::rescan);

	list.set_selection_mode (GTK_SELECTION_SINGLE);
	list.column_titles_passive ();
	list.show ();

	refiller = func;
	refill_arg = arg;
	selected_row = -1;
	selected_column = -1;
}

Gtk_Selector::~Gtk_Selector ()

{
	/* ensure that any row data set with set_row_data_full() is deleted */
	hide_all ();
	list.clear ();
}

void
Gtk_Selector::reset (void (*func)(Gtk_CList &, void *), void *arg)

{
	refiller = func;
	refill_arg = arg;
	selected_row = -1;
	selected_column = -1;

	refill();
}

void
Gtk_Selector::refill ()

{
	if (refiller) {
		list.freeze ();
		list.clear ();
		refiller (list, refill_arg);
		list.thaw ();
	}
}

void
Gtk_Selector::row_selected (gint row, gint col, GdkEvent *ev)

{
	if (ev) { /* && ev->type == GDK_2BUTTON_PRESS) { */
		selected_row = row;
		selected_column = col;
		gtk_idle_add (Gtk_Selector::_accept, this);
	}
}

gint
Gtk_Selector::_accept (gpointer arg)

{
	((Gtk_Selector *) arg)->accept ();
	return FALSE;
}

void
Gtk_Selector::accept ()

{
	_gtk_string buf;
	SelectionResult result;

	if (selected_row >= 0) {
		
		/* XXX if _gtk_string wasn't used here, we could
		   avoid the copy ....
		*/

		if (list.get_text (selected_row, selected_column,&buf)) {
			result.text = new string (buf);
			result.data = list.get_row_data(selected_row);
			result.row = selected_row;
			result.column = selected_column;
			selection_made (this, &result);
		}

	} else {
		cancel ();
	}
}

void
Gtk_Selector::cancel ()

{
	SelectionResult foo;
	foo.text = new string ("");
	foo.data = NULL;

	selection_made (this, &foo);
}

void
Gtk_Selector::rescan ()

{
	selected_row = -1;
	selected_column = -1;
	refill ();
	show_all ();
}

