/*
    Copyright (C) 1999 Paul Barton-Davis 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: selector.h,v 1.3 1999/06/14 16:46:22 pbd Exp $
*/

#ifndef __gtkselector_h__
#define __gtkselector_h__

#include <gtk--.h>

typedef void (SelectorRefillFunction)(Gtk_CList&, void *);

struct SelectionResult {
	int row;
	int column;
        const string *text;
	void *data;

	SelectionResult () {
		row = -1;
		column = -1;
		text = NULL;
		data = NULL;
	}
	
	~SelectionResult() { 
		if (text) delete text;
	}
};

class Gtk_Selector : public Gtk_VBox
{
private:
	Gtk_ScrolledWindow scroll;
	Gtk_CList list;
	void (*refiller)(Gtk_CList &, void *);
	void *refill_arg;
	gint selected_row;
	gint selected_column;

	void row_selected (gint row, gint col, GdkEvent *ev);
	void refill ();
	static gint _accept (gpointer);

public:
	Gtk_Selector (SelectorRefillFunction, void *arg, 
		      size_t columns = 1,
		      bool hscroll = false, 
		      bool vscroll = true,
		      const gchar **titles = 0);
	virtual ~Gtk_Selector ();
	Gtk_CList &clist () { return list; }
	void reset (void (*refiller)(Gtk_CList &, void *), void *arg);
	void set_size (unsigned int w, unsigned int h) {
		scroll.set_usize (w, h);
		list.set_column_width (1, w);
	}
	Signal2<Gtk_Selector *, SelectionResult *> selection_made;
	Signal0 update_contents;

	void accept();
	void cancel();
	void rescan();
};

#endif __gtkselector_h__


